#!/bin/sh -efu
#
# Helper to output man page source file.
#
# Copyright (c) 2008-2009  Alexey Gladkov <gladkov.alexey@gmail.com>
#

. shell-args
. shell-getopt
. man-sh-functions

set_pagesize

show_help() {
	cat <<-EOF
	Usage: $PROG [options] <file> [<line-length>] [<page-length>]

	Options:
	  -l, --line=NUM     specify line length;
	  -p, --page=NUM     specify page length;
	  -c, --config=FILE  specify the configuration file to use;
	                     the default is $config_file.
	  -v, --verbose      print a message for each action;
	  -V, --version      print program version and exit;
	  -h, --help         show this text and exit.

	EOF
	exit
}

print_version() {
	cat <<-EOF
	$PROG version $PROG_VERSION
	Written by Alexey Gladkov <gladkov.alexey@gmail.com>

	Copyright (C) 2008-2009  Alexey Gladkov <gladkov.alexey@gmail.com>
	This is free software; see the source for copying conditions.  There is NO
	warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
	EOF
	exit
}

TEMP=`getopt -n $PROG -o 'c:,l:,p:,h,v,V' -l 'config:,line:,page:,verbose,version,help' -- "$@"` ||
	show_usage
eval set -- "$TEMP"
while :; do
	case "$1" in
		-c|--config) shift; config_file="$1"
			;;
		-l|--line) shift; ll="$1"
			;;
		-p|--page) shift; pl="$1"
			;;
		-h|--help) show_help
			;;
		-V|--version) print_version
			;;
		-v|--verbose) verbose=-v
			;;
		--) shift; break
			;;
		*) fatal "Unrecognized option: $1"
			;;
	esac
	shift
done

[ "$#" -gt 0 ] ||
	show_usage

fn="$(readlink -ev "$1")"; shift
[ "$#" -eq 0 ] || { ll="$1"; shift; }
[ "$#" -eq 0 ] || { pl="$1"; shift; }

verbose "Line length: $ll"
verbose "Page length: ${pl:-None}"

unpackfile() {
	local filename="$1"
	local i=0 suf= cmd=
	while [ "$suffnum" -gt "$i" ]; do
		eval "suf=\"\$suffix$i\""
		if [ -z "${filename##*$suf}" ]; then
			eval "cmd=\"\$uncompress$i\""
			break
		fi
		suf= cmd=
		i=$(($i+1))
	done

	verbose "Checking '$filename'"
	[ -f "$filename" ] ||
		return 1

	verbose "Using '$filename'"
	${cmd:-cat} "$filename" |
		readfile
}

readfile() {
	local l eof=
	while [ -z "$eof" ]; do
		IFS='
'		read -r l || eof=1
		expand_so_directive $l || printf '%s\n' "$l"
	done
}

expand_so_directive() {
	[ "${1-}" = '.so' ] ||
		return 1
	shift

	if [ "$#" -eq 0 ]; then
		message '%s\n' "$fn: .so directive should have argument"
		return 1
	fi
	local mpath manfile="$*"
	verbose "Process command: .so $manfile"

	! unpackfile "$manfile" ||
		return 0

	for mpath in $MANPATH; do
		! unpackfile "$mpath/$manfile" ||
			return 0

		local i=0 suf= cmd=
		while [ "$suffnum" -gt "$i" ]; do
			eval "suf=\"\$suffix$i\""
			! unpackfile "$mpath/$manfile$suf" ||
				return 0
			i=$(($i+1))
		done
	done

	return 1
}

showsource() {
	local l eof= first=
	while [ -z "$eof" ]; do
		IFS='
'		read -r l || eof=1
		if [ -z "$first" ]; then
			case "$l" in
				'.\" -*- mode: troff; coding: '*'-*-')
					printf '%s\n' "$l"
					first=1
					;;
			esac
			if [ "$ll" -gt 0 ]; then
				# We should set line length and title line length.
				# However, a .lt command here fails, only
				#  .ev 1; .lt ...; .ev helps for my version of groff.
				# The LL assignment is needed by the mandoc macros.
				printf '.ll %d.%di\n'    $(($ll/10)) $(($ll%10))
				printf '.nr LL %d.%di\n' $(($ll/10)) $(($ll%10))
				#printf '.lt %d.%di\n'   $(($ll/10)) $(($ll%10))
			fi
			[ -z "$pl" ] ||
				printf '.pl %.128s\n' "$pl"
			[ -n "$first" ] ||
				printf '%s\n' "$l"
			first=1
			continue
		fi
		expand_so_directive $l ||
			printf '%s\n' "$l"
	done
	if [ -n "$first" -a "$pl" = "${MAN_VERY_LONG_PAGE-}" ]; then
	        # At end of the nroff source, set the page length to
		# the current position plus 10 lines.  This plus setpl()
		# gives us a single page that just contains the whole
		# man page. (William Webber, wew@cs.rmit.edu.au)
		printf '.\"\n'
		printf '.pl \\n(nlu+10\n'
	fi
}

read_conf
set_lang
unpackfile "$fn" |showsource
